#
# Python script to plot errors vs. t
#

import os, shutil, glob
from pylab import *
from matplotlib.font_manager import FontProperties

# importing these modules requires subdirectory pyclaw:
from pyclaw import data, controller, error, plotting

refsoln = 'refsoln'

def plotjobs():
    clawjob = controller.ClawController()
    clawjob.xdir = os.getcwd()    # execute xclaw command from current dir.
    
    print "-----------------------------------------------------------------"
    
    t = {}
    err = {}
    tests = [[800,1],[50,3]]

    color = ['b','r','k','g']
    for itest in range(len(tests)):
        figure(itest+1,figsize=(14,6))
        clf()

        mx = tests[itest][0]
        mxnest = tests[itest][1]

        # directory for output:
        outdir = 'output.mx%smxnest%s' % (mx,mxnest) 
        if os.path.isdir(outdir):
            nout = len(glob.glob('%s/fort.q*' % outdir)) - 1
            outdir = os.path.abspath(outdir)
            efile = open(outdir+'/errors.txt','w')
            print "Found directory ",outdir
        
	else:
	    print "*** Could not find directory ",outdir
	    return

        efile.write('\nmx = %5i, levels = %5i' % (mx,mxnest))
        plotdata = data.ClawPlotData()
        plotdata.outdir = outdir
        times = []
        errors = []
        errors_upper = []
        for frameno in range(nout+1):
            t = plotdata.getframe(frameno).t
            errmax,errmax_upper = compute_error(frameno, plotdata)
            times.append(t)
            errors.append(errmax)
            errors_upper.append(errmax_upper)
            efile.write('\n   frameno = %4i, errmax       = %s\n' \
                  % (frameno,errmax[:mxnest]))
            efile.write('\n                  errmax_upper = %s' \
                  % errmax_upper[:mxnest])


        print 'times = ',times
        E = array(errors)
        EU = array(errors_upper)
        EU = where(EU>0., EU, NaN)  # missing values if no fine grid in upper
        #subplot(1,len(tests),itest+1)
        legends = []
        for j in range(mxnest):
            semilogy(times, E[:,j], 'o-', color=color[j])
            legends.append('mx = %s, mxnest = %s, level = %s' % (mx,mxnest,j+1))
            hold(True)
            semilogy(times, EU[:,j], '+-', color=color[j])
            legends.append('only over upper hemisphere')

        tmax = array(times).max()
        xlim([0,2*tmax])
        legend(legends,'best',prop=FontProperties(size='smaller'),\
               markerscale=0.8)
        xlabel('time')
        ylabel('error')
        title('Max norm of error')
        fname = outdir + '/fig7.png'
        savefig(fname)

        efile.close()



    
#------------------------------------

def compute_error(frameno, plotdata):
    from setplot_figure6 import scatter_var
    solution = plotdata.getframe(frameno)
    reference = plotdata.getframe(frameno,refsoln)
    var_2d = 4  # sea surface in 2d calculation
    var_1d = 3  # sea surface in 1d calculation
    errmax,errmax_upper, xs,qs,qint = errors_2d_vs_1d(solution,reference,var_2d, \
                         var_1d,scatter_var)
    return errmax, errmax_upper

#------------------------------------
def errors_2d_vs_1d(solution,reference,var_2d,var_1d,map2d_to_1d):
    """
    Input: 
      solution: object of class ClawSolution with 2d computed solution
      reference: object of class ClawSolution with 1d reference solution
      var_2d: variable to compare from solution
            if integer, compare q[:,:,var_2d]
            if function, apply to q to obtain the variable
      var_1d: variable to compare from reference
            if integer, compare to q[:,var_2d]
            if function, apply to q to obtain the variable
      map2d_to_1d: function mapping 2d data to 1d for comparison,
          signature:  xs, qs = map2d_to_1d(xcenter, ycenter, q)
    """

    from numpy import interp, where

    t = solution.t
    xs = {}
    qs = {}
    qint = {}

    errmax = 5*[0.]        # for mxnest <= 5
    errmax_upper = 5*[0.]
    for gridno in range(len(solution.grids)):
        grid = solution.grids[gridno]

        xc_center = grid.xcenter
        yc_center = grid.ycenter
        [X_center,Y_center] = meshgrid(xc_center,yc_center)

        if grid.mapc2p:
            try:
                X_center,Y_center = grid.mapc2p(X_center,Y_center)
            except:
                print "*** Error with mapc2p ***"
                print "*** in directory: ", os.getcwd()
                traceback.print_exc()
    
        if isinstance(var_2d, int):
            q = grid.q[:,:,var_2d]
        else:
            try:
                q = var_2d(grid.q, X_center, Y_center, t)
            except:
                print '*** Error applying function plot_var = ',plot_var
                traceback.print_exc()
                return 
        
        xs1, qs1 = map2d_to_1d(q, X_center, Y_center, t)

        if hasattr(reference,'grids'):
            if len(reference.grids) > 1:
                print '*** Warning in errors_2d_vs_1d: reference solution'
                print '*** has more than one grid -- only using grid[0]'
            refgrid = reference.grids[0]
        else:
            refgrid = reference   # assume this contains true solution or
                                  # something set separately rather than
                                  # a framesoln
        xref = refgrid.xcenter
        if isinstance(var_1d, int):
            qref = refgrid.q[:,var_1d].T
        else:
            try:
                qref = var_1d(refgrid.q, xref, t)
            except:
                print '*** Error applying function var_1d'
                return 
            
        qint1 = interp(xs1, xref, qref)

        xs[gridno] = xs1
        qs[gridno] = qs1
        qint[gridno] = qint1
        errabs = abs(qs1 - qint1)
        
        j = grid.level - 1
        errmax[j] = max(errmax[j], errabs.max())
        errabs_upper = where(xs1>0, errabs, 0.)
        errmax_upper[j] = max(errmax_upper[j], errabs_upper.max())
        #import pdb
        #pdb.set_trace()

    return errmax, errmax_upper, xs, qs, qint
        
#------------------------------------

if __name__=='__main__':
    # when executed from command line, call main routine:
    plotjobs()
    print "Next execute make_figure6.py"

