
def mapc2m(xc,yc):
    """
    Map [-3,1] x [-1,1] to sphere of radius r1, as described in
        Logically Rectangular Grids and Finite Volume Methods for PDEs
             in Circular and Spherical Domains,
        by Donna A. Calhoun, Christiane Helzel, and Randall J. LeVeque,
        http://www.amath.washington.edu/~rjl/pubs/circles
    """

    from numpy import sqrt, abs, where, sign, ones, pi, sin
    
    Rsphere = 6.371e6
    r1 = Rsphere
    sgnz = where(xc<-1, -1, 1)
    xc = where(xc<-1., -2.-xc, xc)
    absxc = abs(xc)
    absyc = abs(yc)
    d = where(absxc>absyc, absxc, absyc)
    d = where(d>1e-14, d, 1e-14)  # to avoid divide by zero

    D = r1 * d*(2.-d)/sqrt(2)
    D = r1 * sin(pi*d/2)/sqrt(2)
    R = r1 * ones(d.shape)

    center = D - sqrt(R**2 - D**2)
    xp = D/d * abs(xc)
    yp = D/d * abs(yc)

    yp = where(abs(yc)>=abs(xc), center + sqrt(R**2 - xp**2), yp)
    xp = where(abs(xc)>=abs(yc), center + sqrt(R**2 - yp**2), xp)

    xp = sign(xc) * xp
    yp = sign(yc) * yp
    zp = sgnz * sqrt(r1**2 - (xp**2 + yp**2))

    return xp,yp,zp
