"""
Module Iplotclaw for interactive plotting of Clawpack results.

Starting to modify for pyclaw.

"""

import cmd, os, sys, string

try:
    from numpy import *
except:
    print "problem importing numpy"
    sys.exit(1)

if not sys.modules.has_key('matplotlib'):
    try:
        import matplotlib
        # Override system defaults before importing pylab
        #matplotlib.use('Agg')  # Use an image backend
        matplotlib.use('TkAgg')
        matplotlib.rc('text', usetex=False)
        matplotlib.interactive(True)

    except:
        print "problem importing matplotlib"
        sys.exit(1)

try:
    from pylab import *
except:
    print "problem importing pylab"
    sys.exit(1)

from pyclaw import data, plotting

#rundata = data.ClawData('claw.data')

#------------------------
class Iplotclaw(cmd.Cmd):
#------------------------
    """
    Class for interactively stepping through Clawpack plots.
    Uses plotting.plotframe to actually plot each frame.

    Usage:
    ------
    >>> from pyclaw.Iplotclaw import Iplotclaw 
    >>> pc = Iplotclaw()             # new instantiation
    >>> pc.plotloop()                # to start looping
    PLOTCLAW > help                  # for list of available commands
    PLOTCLAW > q                     # to quit looping and return to python
    >>> pc.plotloop()                # to restart looping at previous frame,
                                     #    with data, outdir, etc. preserved.
    """
    from pyclaw import plotting, data

    # initialization:
    # ---------------

    prompt = 'PLOTCLAW > '
    lastcmd = 'n'             # initialize so <return> advances frame'

    def __init__(self, setplot='setplot.py', \
                 completekey='tab', stdin=None, stdout=None):
        """Instantiate a line-oriented interpreter framework.


        The optional argument 'completekey' is the readline name of a
        completion key; it defaults to the Tab key. If completekey is
        not None and the readline module is available, command completion
        is done automatically. The optional arguments stdin and stdout
        specify alternate input and output file objects; if not specified,
        sys.stdin and sys.stdout are used.

        """
        import sys
        if stdin is not None:
            self.stdin = stdin
        else:
            self.stdin = sys.stdin
        if stdout is not None:
            self.stdout = stdout
        else:
            self.stdout = sys.stdout
        self.cmdqueue = []
        self.completekey = completekey
 
        self.setplot = setplot


    def preloop(self):
        #from pyclaw import plotting, data
        #ion()
        #self.completekey = None
        try:
	    prevframeno = self.frameno
            prevplotdata = self.plotdata
            plotdata = self.plotdata
	    restart = True
	except:
	    prevframeno = 0
            plotdata = data.ClawPlotData()
            plotdata.setplot = self.setplot
            prevplotdata = plotdata
	    restart = False
            print 'New start'

        plotdata._mode = 'iplotclaw'

        if not restart:
            try:
                plotdata = plotting.call_setplot(self.setplot,plotdata)
            except:
                print '*** Problem executing setplot'
		print '    plotdata = ', plotdata
                print '    setplot = ', self.setplot
		raise
                return

        print '\nInteractive plotting for Clawpack output... '
        print '\nPlotting data from outdir = ', plotdata.outdir
        print 'Type ? at PLOTCLAW prompt for list of commands'

        startframeno = raw_input('\n    Start at which frame [default=%i] ? '\
	                            % prevframeno)

	makeplot = True 


	if startframeno == '':
	    self.frameno = prevframeno
	    self.plotdata = prevplotdata
	    if restart:
	        replot = raw_input('    Replot data for frame %s [no] ? ' \
	                            % self.frameno)

	        if replot not in ['y','yes','Y']:
	            makeplot = False

		if makeplot:
	            reload = raw_input('    Reload data for frame %s [no] ? ' \
	                                % self.frameno)
	            if reload in ['y','yes','Y']:
	                self.plotdata.refresh_frames = True
                    else:
	                self.plotdata.refresh_frames = False
        else:
	    self.plotdata = prevplotdata
	    try:
	        self.frameno = int(startframeno)
	    except:
                print '\n    *** Error: frameno must be an integer, resetting to 0'
	        self.frameno = 0
        

	if makeplot:
	    plotting.plotframe(self.frameno, self.plotdata)
            self.plotdata.refresh_frames=False
            #draw()

	self.lastcmd = 'n'

    # end of initialization 
    # ---------------------

    def postloop(self):
        #self.framedata = self.plotdata.getframe(self.frameno)
	#grid0 = self.framedata.grids[0]
        #self.q = grid0.q
	#self.mx = grid0.mx
	#if self.ndim > 1:
	#    self.my = grid0.my
        pass


    # Commands that can be typed at the PLOTCLAW> prompt:
    # ---------------------------------------------------

    # help command:
    # -------------
    def help_help(self):
        print 'print this list of valid commands\n'

    # next frame:
    # -----------
    def do_n(self, rest):
        from pyclaw import plotting, data
        #print '    frameno = ',self.frameno
        self.frameno = self.frameno+1
	plotting.plotframe(self.frameno, self.plotdata)
        draw()
    def help_n(self):
        print 'n: advance to next frame\n'

    # previous frame:
    # ---------------
    def do_p(self, rest):
        #print '    frameno = ',self.frameno
        self.frameno = max(self.frameno-1, 0)
	plotting.plotframe(self.frameno, self.plotdata)
    def help_p(self):
        print 'p: go back to previous frame\n'

    # jump to arbitrary frame:
    # ------------------------
    def do_j(self, rest):
        try:
            newframeno = int(rest)
        except:
            newframeno = raw_input('\n    Jump to which frame? ')
        if newframeno == 'n': 
            self.do_n(rest)
        elif newframeno == 'p': 
            self.do_p(rest)
        else:
	    try:
	        newframeno = int(newframeno)
	        self.frameno = newframeno
	        plotting.plotframe(self.frameno, self.plotdata)
	    except:
                print '\n    *** Error: frameno must be an integer, n, or p'
    def help_j(self):
        print 'j N: jump to frame N\n'
        print 'j:   jump to some other frame (will prompt for N)\n'

    # redraw frame:
    # -------------
    def do_r(self, rest):
	plotting.plotframe(self.frameno, self.plotdata)
    def help_r(self):
        print 'r: redraw the current frame,  rr: reload and redraw\n'

    def do_rr(self, rest):
        #self.plotdata.refresh_frames=True
        xxx = self.plotdata.framesoln_dict.pop(self.frameno,None)
        if xxx is None:
           print 'No frame data to clear for frame ',self.frameno
        else:
           print 'Cleared data for frame ',self.frameno
        print 'Reading data from outdir = ',self.plotdata.outdir
	plotting.plotframe(self.frameno, self.plotdata)
        self.plotdata.refresh_frames=False
    def help_rr(self):
        print 'r: redraw the current frame,  rr: reload and redraw\n'

    # call setplot again
    # --------------------
    def do_resetplot(self, rest):
        if rest:
            self.setplot = rest
            print '*** Resetting setplot to: ',rest
            self.plotdata.setplot = self.setplot
        try:
            plotdata = plotting.call_setplot(self.setplot,self.plotdata)
        except:
            print '*** Problem re-executing setplot'

    def help_resetplot(self):
        print 'resetplot: re-execute the function setplot'
        print '           The easiest way to change plotting parameters'
        print '           is to modify setplot.py and then do resetplot.'
        print ' '
        print 'resetplot <new>: switch to a different setplot function'
        print '           as specified by <new>, which is a function or'
        print '           a string specifying the module containing setplot.'

    # show plot parameters:
    # ---------------------
    def do_show(self, rest):
        self.plotdata.showitems()

    def help_show(self):
        print 'show: show the current plot items'

    # clearframes
    # ---------
    def do_clearframes(self, rest):
	if rest=='':
            self.plotdata.framesoln_dict.clear()
            print 'Cleared all frames'
	else:
            try:
                frameno = int(rest)
                xxx = self.plotdata.framesoln_dict.pop(frameno,None)
                if xxx is None:
                   print 'No frame data to clear for frame ',frameno
                else:
                   print 'Cleared data for frame ',frameno
            except:
                print 'Must specify a frame number (integer)'

    def help_clearframes(self):
        print 'clearframes: delete frame data from cache to replot'
        print '    use if you have rerun the code and want to plot the'
        print '    latest results'
        print '          clearframes frameno  clears one frame'
        print '          clearframes          clears all frames'


    # print working directory:
    # ------------------------
    def do_pwd(self, rest):
        print '  now in directory: ',os.getcwd()
        print '  data from outdir: ',self.plotdata.outdir
    def help_pwd(self):
        print 'pwd: print current working directory and outdir'
        print '     fort.* files in outdir provide frame data\n'



    # print figure to a file:
    # -----------------------
    def do_print(self, rest):
        #from pyclaw import plotting
	fname = rest
        try:
            for figno in self.plotdata.iplotclaw_fignos:
                if len(fname)>0:
	            plotting.printfig(fname, self.frameno, figno)
                else:
	            plotting.printfig(frameno=self.frameno, figno=figno)
	except:
	    print '    *** Error saving figure to file'
    def help_print(self):
        print 'print: print the figure to file frame000N.png'
        print 'print fname: print the figure to file fname or fname.png\n'
        
    # use vi e.g. to edit setplot.py:
    # --------------
    def do_vi(self, rest):
        os.system('vi %s' % rest)
    def help_vi(self):
        print 'use vi, for example to change the plot parameters: \n'
        print '    vi setplot.py \n'
        print '    resetplot \n'
        

    # quit commands:
    # --------------
    def do_quit(self, rest):
        print 'quitting...'
	return True
    def help_quit(self):
        print 'q or quit: terminates the command loop\n'
        
    def do_q(self, rest):
        print 'quitting...'
	return True
    def help_q(self):
        print 'q or quit: terminates the command loop\n'
        
    def do_k(self, rest):
        print 'quitting...'
	return True
    def help_k(self):
        print 'k: terminates the command loop\n'
        
    # alias plotloop = cmdloop:
    # -------------------------
    def plotloop(self):
        self.cmdloop()

# end of Iplotclaw.
#----------------------------------------------------------------------

