import os,sys,shutil,glob
import string,re
import time
import traceback


try:
    from numpy import *
except:
    '*** Error: NumPy not found'

from data import ClawData

plotter = 'matplotlib'
if plotter == 'matplotlib':
    if not sys.modules.has_key('matplotlib'):
        try:
            import matplotlib
            matplotlib.use('Agg')  # Use an image backend
        except:
            print "*** Error: problem importing matplotlib"

try:
    from pylab import *
except:
    print "*** Error: problem importing pylab"



#==========================================
def plotframe(frameno, plotdata, verbose=False):
#==========================================

    """
    Plot a single frame from the computation.

    This routine checks input and then calls plotframeN for the
    proper space dimension.

    """

    if verbose:  print '    Plotting frame %s ... '  % frameno

    ion()
    #if plotdata.ion:
        #ion()
    #else:
        #ioff()

        
    try:
        plotfigure_dict = plotdata.plotfigure_dict
    except:
        print '*** Error in plotframe: plotdata missing plotfigure_dict'
        print '*** This should not happen'
        return 

    if len(plotfigure_dict) == 0:
        print '*** Warning in plotframe: plotdata has empty plotfigure_dict'
        print '*** Apparently no figures to plot'


    try:
        framesoln = plotdata.getframe(frameno)
        t = framesoln.t
    except:
        print '*** Cannot find frame number ',frameno
        print '*** looking in directory ', os.getcwd()
        return


    # iterate over each single plot that makes up this frame:
    # -------------------------------------------------------
 
    if plotdata._mode == 'iplotclaw':
        requested_fignos = plotdata.iplotclaw_fignos
    else:
        requested_fignos = plotdata.print_fignos
    plotted_fignos = []
    #print '+++ mode = ', plotdata._mode
    #print '+++ requested_fignos = ', requested_fignos

    plotdata = set_show(plotdata)   # set _show attributes for which figures
                                    # and axes should be shown.

    # loop over figures to appear for this frame: 
    # -------------------------------------------

    for figname in plotdata._fignames:
        plotfigure = plotdata.plotfigure_dict[figname]
        if not plotfigure._show:
            continue  # skip to next figure 

        figno = plotfigure.figno
        if requested_fignos != 'all':
            if figno not in requested_fignos:
                continue # skip to next figure

        plotted_fignos.append(figno)


	if not plotfigure.kwargs.has_key('facecolor'):
	    # use Clawpack's default bg color (tan)
	    plotfigure.kwargs['facecolor'] = '#ffeebb'   

        # create figure and set handle:
        plotfigure._handle = figure(num=figno, **plotfigure.kwargs)

        ioff()
        if plotfigure.clf_each_frame:
            clf()

        #print '+++ working on figure ',figno

        try:
            plotaxes_dict = plotfigure.plotaxes_dict
        except:
            print '*** Error in plotframe: plotdata missing plotaxes_dict'
            print '*** This should not happen'
            return 

        if (len(plotaxes_dict) == 0) or (len(plotfigure._axesnames) == 0):
            print '*** Warning in plotframe: plotdata has empty plotaxes_dict'
            print '*** Apparently no axes to plot in figno ',figno

        # loop over axes to appear on this figure:
        # ----------------------------------------

        for axesname in plotfigure._axesnames:
            plotaxes = plotaxes_dict[axesname]
            if not plotaxes._show:
                continue   # skip this axes if no items show

            # create the axes:
            axescmd = getattr(plotaxes,'axescmd','subplot(1,1,1)')
            exec('plotaxes._handle = %s' % axescmd)
            hold(True)

            # loop over plotitems on this set of axes:
            # ----------------------------------------

            for itemname in plotaxes._itemnames:
                
                plotitem = plotaxes.plotitem_dict[itemname]
                if plotitem._show == False:
                    print 'Skipping item ', itemname
                    continue

                # Get solution for this frame:
                try:
                    framesoln = plotdata.getframe(frameno, plotitem.outdir)
                    t = framesoln.t
                except:
                    print '*** Cannot find frame number ',frameno
                    if plotitem.outdir:
                        print '*** outdir =  ',plotitem.outdir
                    return


                # call plotitemN to create a single plot in dimension N=ndim:
                # -------------------------------------------------------------

                ndim = plotitem.ndim
                cmd = 'plotitem%s(framesoln,plotitem)'  % ndim

                try:
                    exec(cmd)
                    if verbose:  
                            print '      Plotted  plotitem ', itemname
                except:
                    print '*** Error in plotframe: problem calling plotitem%s' % ndim
                    traceback.print_exc()
                    return

                # end of loop over plotitems


            # set axes limits:
            if (plotaxes.xlim is not None) & (type(plotaxes.xlim) is not str):
                try:
                    plotaxes._handle.set_xlim(plotaxes.xlim[0], plotaxes.xlim[1])
                except:
                    pass  # let axis be set automatically
            if (plotaxes.ylim is not None) & (type(plotaxes.ylim) is not str):
                try:
                    plotaxes._handle.set_ylim(plotaxes.ylim[0], plotaxes.ylim[1])
                except:
                    pass  # let axis be set automatically

            if plotaxes.afteraxes:
                if isinstance(plotaxes.afteraxes, str):
                    exec(plotaxes.afteraxes)
                else:
                    # assume it's a function
                    try:
                        plotaxes.afteraxes()
                    except:
                        '*** Warning: could not execute afteraxes'

            if plotaxes.title_with_t:
                title("%s at time %8.4f" % (plotaxes.title,t))
            else:
                title(plotaxes.title)

            # end of loop over plotaxes
            
        # end of loop over plotfigures


    # call an afterframe function if desired:
    afterframe =  getattr(plotdata, 'afterframe', None)
    if afterframe:
        try:
            afterframe(frameno, plotdata)
        except:
            print '*** Error in afterframe ***'
            traceback.print_exc()

    ion()
    for figno in plotted_fignos:
        figure(figno)
        #print '+++ drawing figure ',figno
        draw()

    if verbose:
        print '    Done with plotframe for frame %i at time %g' % (frameno,t)

    
    # print the figure(s) to file(s) if requested:
    if (plotdata.mode() != 'iplotclaw') & plotdata.printfigs:
        # iterate over all figures that are to be printed:
        for figno in plotted_fignos:
            printfig(frameno=frameno, figno=figno, \
                    format=plotdata.print_format, plotdir=plotdata.plotdir,\
                    verbose=verbose)



    
#==================================================================
def plotitem1(framesoln, plotitem):
#==================================================================
    """
    Make a 1d plot for a single plot item for the solution in framesoln.

    """

    plotdata = plotitem._plotdata
    plotfigure = plotitem._plotfigure
    plotaxes = plotitem._plotaxes

    for gridno in range(len(framesoln.grids)):
        grid = framesoln.grids[gridno]

        t = framesoln.t
    
        # the following plot parameters should now be set:
    
        try:
            plot_type = plotitem.plot_type
            plot_var = plotitem.plot_var
            aftergrid = plotitem.aftergrid
            plotstyle = plotitem.plotstyle
            color = plotitem.color
            kwargs = plotitem.kwargs
        except:
            print '*** Error in plotitem1:' + \
                  '  plotitem missing required attribute'
            print 'plotitem = ',plotitem
            traceback.print_exc()
            return 
        
        if plot_type == '1d':
	    thisgridvar = get_gridvar(grid,plot_var,ndim=1)
	    X_center = thisgridvar.X_center   # cell centers
	    var = thisgridvar.var             # variable to be plotted

        elif plot_type == '1d_from_2d_data':
            map_2d_to_1d =  getattr(plotitem, 'map_2d_to_1d', None)
            if not map_2d_to_1d:
                print '*** Error, plot_type = 1d_from_2d_data requires '
                print '*** map_2d_to_1d function as plotitem attribute'
                raise
                return
            try:
	        thisgridvar = get_gridvar(grid,plot_var,ndim=2)
	        X_center = thisgridvar.X_center   # cell centers
	        Y_center = thisgridvar.Y_center
	        var = thisgridvar.var             # variable to be plotted
                X_center, var = map_2d_to_1d(var,X_center,Y_center,t)
            except:
                print '*** Error with map_2d_to_1d function'
                print 'map_2d_to_1d = ',plotitem.map_2d_to_1d
                raise
                return

    
        # The plot commands using matplotlib:
    
        hold(True)

        if  isinstance(color, list):
            color2 = color[grid.level-1]
        else:
            color2 = color

        if  isinstance(plotstyle, list):
            plotstyle2 = plotstyle[grid.level-1]
        else:
            plotstyle2 = plotstyle

        #kwargs = getattr(plotitem,'kwargs',{})
        #kwargs['color'] = color2

        if plotstyle != '':
            #plotcommand = "plot(X_center,var,'%s', **kwargs)"  \
                          #% plotstyle2
            plotcommand = "plot(X_center,var,'%s', color='%s', **kwargs)"  \
                          % (plotstyle2, color2)
            exec(plotcommand)
    
    
        if aftergrid:
            try:
                if isinstance(aftergrid, str):
                    exec(aftergrid)
                else:
                    # assume it's a function
                    aftergrid(gridno, frameno, plotitem)
            except:
                print '*** Warning: could not execute aftergrid'

        # end of loop over grids

    return  
    

#==================================================================
def plotitem2(framesoln, plotitem):
#==================================================================
    """
    Make a 2d plot for a single plot item for the solution in framesoln.

    """

    from pyclaw.colormaps import white   

    for gridno in range(len(framesoln.grids)):
        grid = framesoln.grids[gridno]

        t = framesoln.t
    
    
        # the following plot parameters should now be set:
    
        try:
            plot_var = plotitem.plot_var
            aftergrid = plotitem.aftergrid
            plot_type = plotitem.plot_type
            plot_grid = plotitem.plot_grid
            grid_color = plotitem.grid_color
            plot_grid_edges = plotitem.plot_grid_edges
            grid_edge_color = plotitem.grid_edge_color
        except:
            print '*** Error in plotitem2:' + \
                  '  plotitem missing required attribute'
            print 'plotitem = ',plotitem
            traceback.print_exc()
            return
    
        #print 'plot_type = ',plot_type
    
        if plot_type == '2d_pcolor':
            try:
                pcolor_cmap = plotitem.pcolor_cmap
                pcolor_cmin = plotitem.pcolor_cmin
                pcolor_cmax = plotitem.pcolor_cmax
                pcolor_colorbar = plotitem.pcolor_colorbar
            except:
                print '*** Error in plotitem2:' + \
                      '  plotitem missing required attribute'
                print 'plotitem = ',plotitem
                traceback.print_exc()
                return
    
        elif plot_type == '2d_contour':
            try:
                contour_nlevels = plotitem.contour_nlevels
                contour_levels = plotitem.contour_levels
                contour_min = plotitem.contour_min
                contour_max = plotitem.contour_max
                contour_kwargs = plotitem.contour_kwargs
            except:
                print '*** Error in plotitem2:' + \
                      '  plotitem missing required attribute'
                print 'plotitem = ',plotitem
                traceback.print_exc()
                return

        level = grid.level

        if isinstance(plot_grid, list):
            plot_grid = plot_grid[level-1]
        if not isinstance(plot_grid, int):
            print '*** Error in plotitem2: plot_grid should be int here'

        if isinstance(plot_grid_edges, list):
            plot_grid_edges = plot_grid_edges[level-1]
        if not isinstance(plot_grid_edges, int):
            print '*** Error in plotitem2: plot_grid_edges should be int here'

        if isinstance(grid_color, list):
            grid_color = grid_color[level-1]
        if isinstance(grid_edge_color, list):
            grid_edge_color = grid_edge_color[level-1]

        
	thisgridvar = get_gridvar(grid,plot_var,ndim=2)

	X_center = thisgridvar.X_center   # cell centers (on mapped grid)
	Y_center = thisgridvar.Y_center
	X_edge = thisgridvar.X_edge       # cell edges (on mapped grid)
	Y_edge = thisgridvar.Y_edge
	var = thisgridvar.var             # variable to be plotted
    
    
    
        # The plot commands using matplotlib:
    
        hold(True)
    
        #print '+++ q.shape = ',q.shape

        #import pdb
        #pdb.set_trace()

        if plot_type == '2d_pcolor':
            if plot_grid:
                pobj = pcolor(X_edge, Y_edge, transpose(var), \
                    cmap=pcolor_cmap, edgecolors=grid_color, shading='faceted')
            else: 
                pobj = pcolor(X_edge, Y_edge, transpose(var), \
                    cmap=pcolor_cmap, edgecolors='None', shading='flat')
                #print '+++ pcolor flat for gridno = ',gridno
            clim(pcolor_cmin, pcolor_cmax) 


        elif plot_type == '2d_contour':
            levels_set = True
            if contour_levels is None:
                levels_set = False
                if contour_nlevels is None:
                    print '*** Error in plotitem2:'
                    print '    contour_levels or contour_nlevels must be set'
                    raise
                    return
                if contour_min and contour_max:
                    contour_levels = linspace(contour_min,contour_max,\
                                 contour_nlevels)
                    levels_set = True 
            # attempt to white out underlying contour lines from 
            # coarser grids.  This does not work yet!! *** FIX! ***
            #pcolor(X_edge, Y_edge, transpose(var), \
                    #cmap=white(), edgecolors='None', shading='flat')

            if plot_grid:
                pobj = pcolor(X_edge, Y_edge, zeros(var.T.shape), \
                        cmap=white(), edgecolors=grid_color, shading='faceted')
            else: 
                pobj = pcolor(X_edge, Y_edge, zeros(var.T.shape), \
                        cmap=white(), edgecolors='None', shading='flat')
            hold(True)


            # create the contour command:
            contourcmd = "contour(X_center, Y_center, transpose(var), "
            if levels_set:
                contourcmd += "contour_levels"
            else:
                contourcmd += "contour_nlevels"
            if contour_kwargs:
                contourcmd += ", **contour_kwargs)"
            else:
                contourcmd += ")"
            exec(contourcmd)

        elif plot_type == '2d_grid':
            # plot only the grids, no data:
            # Should be improved to allow different bg color for each level!
            if plot_grid:
                pobj = pcolor(X_edge, Y_edge, zeros(var.T.shape), \
                        cmap=white(), edgecolors=grid_color, shading='faceted')
            else: 
                pobj = pcolor(X_edge, Y_edge, zeros(var.T.shape), \
                        cmap=white(), edgecolors='None', shading='flat')
    


        if plot_grid:
            # needed for contour plots or different color edges
            for i in range(X_edge.shape[0]):
                plot(X_edge[i,:], Y_edge[i,:], grid_color)
            for i in range(X_edge.shape[1]):
                plot(X_edge[:,i], Y_edge[:,i], grid_color)

        if plot_grid_edges:
            for i in [0, X_edge.shape[0]-1]:
                X1 = X_edge[i,:]
                Y1 = Y_edge[i,:]
                plot(X1, Y1, grid_edge_color)
            for i in [0, X_edge.shape[1]-1]:
                X1 = X_edge[:,i] 
                Y1 = Y_edge[:,i]
                plot(X1, Y1, grid_edge_color)

    
        if aftergrid:
            try:
                if isinstance(aftergrid, str):
                    exec(aftergrid)
                else:
                    # assume it's a function
                    aftergrid(gridno, frameno, plotitem)
            except:
                print '*** Warning: could not execute aftergrid'

        # end of loop over grids


    if plot_type == '2d_pcolor':
        if pcolor_colorbar:
            colorbar(pobj)

    return 


#--------------------------------------
def get_gridvar(grid, plot_var, ndim):
#--------------------------------------
    """
    Return arrays for spatial variable(s) (on mapped grid if necessary)
    and variable to be plotted on a single grid in ndim space dimensions.
    """
    
    if ndim == 1:
    
        if grid.mapc2p:
            try:
                X_center = grid.mapc2p(grid.xcenter)
                X_edge = grid.mapc2p(grid.xedge)
            except:
                print "*** Error with mapc2p ***"
                traceback.print_exc()
                return  
        else:
            X_center = grid.xcenter
            X_edge = grid.xedge
    
        if isinstance(plot_var, int):
            var = grid.q[:,plot_var]
        else:
            try:
                var = plot_var(grid.q, X_center, grid.t)
            except:
                print '*** Error applying function plot_var = ',plot_var
                traceback.print_exc()
                return  

	thisgridvar = ClawData()
	thisgridvar.var = var
	thisgridvar.X_center = X_center
	thisgridvar.X_edge = X_edge
	thisgridvar.mx = grid.mx

	# end of 1d case
    
    elif ndim == 2:
        xc_center = grid.xcenter
        yc_center = grid.ycenter
        [X_center,Y_center] = meshgrid(xc_center,yc_center)

        xc_edge = grid.xedge
        yc_edge = grid.yedge
        [X_edge,Y_edge] = meshgrid(xc_edge,yc_edge)

        if grid.mapc2p:
            try:
                X_center,Y_center = grid.mapc2p(X_center,Y_center)
                X_edge,Y_edge = grid.mapc2p(X_edge,Y_edge)
            except:
                print "*** Error with mapc2p ***"
                print "*** in directory: ", os.getcwd()
                traceback.print_exc()

        if isinstance(plot_var, int):
            var = grid.q[:,:,plot_var]
        else:
            try:
                var = plot_var(grid.q, X_center, Y_center, grid.t)
            except:
                print '*** Error applying function plot_var = ',plot_var
                traceback.print_exc()
                return 

	thisgridvar = ClawData()
	thisgridvar.var = var
	thisgridvar.X_center = X_center
	thisgridvar.Y_center = Y_center
	thisgridvar.X_edge = X_edge
	thisgridvar.Y_edge = Y_edge
	thisgridvar.mx = grid.mx
	thisgridvar.my = grid.my

	# end of 2d case

    return thisgridvar


#------------------------------------------------------------------------
def printfig(fname='',frameno='', figno='', format='png', plotdir='.', \
             verbose=True):
#------------------------------------------------------------------------
    """
    Save the current plot to file fname or standard name from frame/fig.
.  
    If fname is nonempty it is used as the filename, with extension
    determined by format if it does not already have a valid extension.

    If fname=='' then save to file frame000NfigJ.ext  where N is the frame
    number frameno passed in, J is the figure number figno passed in,
    and the extension ext is determined by format.  
    If figno='' then the figJ part is omitted.
    """

    if fname == '':
        fname = 'frame' + str(frameno).rjust(4,'0') 
        if isinstance(figno,int):
            fname = fname + 'fig%s' % figno
    splitfname = os.path.splitext(fname)
    if splitfname[1] not in ('.png','.emf','.eps','.pdf'):
        fname = splitfname[0] + '.%s' % format
    if figno=='':
        figno = 1
    figure(figno)
    if plotdir != '.':
       fname = os.path.join(plotdir,fname)
    if verbose:  print '    Saving plot to file ', fname
    savefig(fname)



#======================================================================
def printframes(plotdata=None, verbose=True):
#======================================================================

    """
    Produce a set of png files for all the figures specified by plotdata.
    Also produce a set of html files for viewing the figures and navigating
    between them.  These will all be in directorey plotdata.plotdir.

    The ClawPlotData object plotdata will be initialized by a call to
    function setplot unless plotdata.setplot=False.  

    If plotdata.setplot=True then it is assumed that the current directory
    contains a module setplot.py that defines this function.

    If plotdata.setplot is a string then it is assumed this is the name of
    a module to import that contains the function setplot.

    If plotdata.setplot is a function then this function will be used.
    """

    import glob
    from pyclaw.data import ClawPlotData


    if not sys.modules.has_key('matplotlib'):
        print '*** Error: matplotlib not found, no plots will be done'
        return plotdata
        
    if not isinstance(plotdata,ClawPlotData):
        print '*** Error, plotdata must be an object of type ClawPlotData'
        return plotdata

    plotdata._mode = 'printframes'

    plotdata = call_setplot(plotdata.setplot, plotdata)

    try:
        plotdata.rundir = os.path.abspath(plotdata.rundir)
        plotdata.outdir = os.path.abspath(plotdata.outdir)
        plotdata.plotdir = os.path.abspath(plotdata.plotdir)

        framenos = plotdata.print_framenos # frames to plot
        fignos = plotdata.print_fignos     # figures to plot at each frame
        fignames = {}                      # names to use in html files

        rundir = plotdata.rundir       # directory containing *.data files
        outdir = plotdata.outdir       # directory containing fort.* files
        plotdir = plotdata.plotdir     # where to put png and html files
        overwrite = plotdata.overwrite # ok to overwrite?
        msgfile = plotdata.msgfile     # where to write error messages
    except:
        print '*** Error in printframes: plotdata missing attribute'
        print '  *** plotdata = ',plotdata
        return plotdata

    if fignos == 'all':
        fignos = plotdata._fignos
        #for (figname,plotfigure) in plotdata.plotfigure_dict.iteritems():
        #    fignos.append(plotfigure.figno)

    rootdir = os.getcwd()

    # annoying fix needed when EPD is used for plotting under cygwin:
    if rootdir[0:9] == 'C:\cygwin' and outdir[0:9] != 'C:\cygwin':
        outdir = 'C:\cygwin' + outdir
        plotdata.outdir = outdir
    if rootdir[0:9] == 'C:\cygwin' and rundir[0:9] != 'C:\cygwin':
        rundir = 'C:\cygwin' + rundir
        plotdata.rundir = rundir
    if rootdir[0:9] == 'C:\cygwin' and plotdir[0:9] != 'C:\cygwin':
        plotdir = 'C:\cygwin' + plotdir
        plotdata.plotdir = plotdir

    try:
        os.chdir(rundir)
    except:
        print '*** Error: cannot move to run directory ',rundir
        print 'rootdir = ',rootdir
        return plotdata

    if msgfile != '':
        sys.stdout = open(msgfile, 'w')
        sys.stderr = sys.stdout


    if os.path.isfile(plotdir):
        print "Error: plotdir specified is a file"
        return plotdata
    elif (os.path.isdir(plotdir) & overwrite):
        if verbose:
            print "Directory ", plotdir, " already exists, "
            print "   png and html files will be overwritten "
    elif (os.path.isdir(plotdir) & (not overwrite)):
        print "Error in plotitem1: Directory ", plotdir, " already exists"
        print "  Remove directory with 'rm -r ",plotdir,"' and try again,"
        print "  or use overwrite=True in call to printframes"
        return plotdata
    else:
        try:
            os.mkdir(plotdir)
        except:
            print "Cannot make directory ",plotdir
            return plotdata

    try:
        os.chdir(plotdir)
    except:
        print '*** Error in printframes: cannot move to plotdir = ',\
              plotdir
        raise
        return plotdata

    framefiles = glob.glob(os.path.join(plotdir,'frame*.png')) + \
                    glob.glob(os.path.join(plotdir,'frame*.html'))
    if overwrite:
        # remove any old versions:
        for file in framefiles:
            os.remove(file)
    else:
        if len(framefiles) > 1:
            print "*** Remove frame*.png and frame*.html and try again,"
            print "  or use overwrite=True in call to printframes"
            return plotdata

    
    # Create each of the figures
    #---------------------------

    try:
        os.chdir(outdir)
    except:
        print '*** Error printframes: cannot move to outdir = ',outdir
        return plotdata


    fortfile = {}
    pngfile = {}
    frametimes = {}

    import glob
    for file in glob.glob('fort.q*'):
        frameno = int(file[7:10])
        fortfile[frameno] = file
        for figno in fignos:
            pngfile[frameno,figno] = 'frame' + file[-4:] + 'fig%s.png' % figno
    
    if len(fortfile) == 0:
        print '*** No fort.q files found in directory ', os.getcwd()
        return plotdata
    
    # Discard frames that are not from latest run, based on
    # file modification time:
    framenos = only_most_recent(framenos, plotdata.outdir)

    numframes = len(framenos)

    print "Will plot %i frames numbered:" % numframes, framenos
    print 'Will make %i figure(s) for each frame, numbered: ' % len(fignos),\
          fignos

    fignames = {}
    for figname in plotdata._fignames:
        figno = plotdata.plotfigure_dict[figname].figno
        fignames[figno] = figname

    #print '+++ fignames = ',fignames

    
    # Make png files by calling plotframe:
    # ------------------------------------

    for frameno in framenos:
        #plotframe(frameno, plotdata, verbose)
        frametimes[frameno] = plotdata.getframe(frameno).t
        #print 'Frame %i at time t = %s' % (frameno, frametimes[frameno])

    os.chdir(plotdir)

    if plotdata.html:
        print 'Making html pages...'
        msg_html = pngfigs2html(plotdata,framenos,frametimes,fignos,fignames)
    else:
        msg_html = ''

    if plotdata.latex:
        print 'Making latex file ...'
        msg_latex = pngfigs2latex(plotdata,framenos,frametimes,fignos,fignames)
    else:
        msg_latex = ''

    if not plotdata.printfigs:
        print "Using previously printed figure files"
    else:
        print "Now making png files for all figures..."
        for frameno in framenos:
            plotframe(frameno, plotdata, verbose)
            #frametimes[frameno] = plotdata.framesoln_dict[frameno].t
            print 'Frame %i at time t = %s' % (frameno, frametimes[frameno])

    if plotdata.latex & plotdata.latex_pdf:
        try:
            os.system('pdflatex %s' % plotdata.latex_fname)
        except:
            print '*** pdflatex command failed in printframes'
    
    print msg_html
    print msg_latex


    # Movie:
    #-------
    
    if plotdata.gif_movie:
        print 'Making gif movies.  This may take some time....'
        for figno in fignos:
            try:
                os.system('convert -delay 20 frame*fig%s.png moviefig%s.gif' \
                   % (figno,figno))
                print '    Created moviefig%s.gif' % figno
            except:
                print '*** Error creating moviefig%s.gif' % figno
    
    os.chdir(rootdir)
    # reset stdout for future print statements
    sys.stdout = sys.__stdout__

    return plotdata
    # end of printframes

#======================================================================
def plotclaw2html(plotdata=None, verbose=True):
#======================================================================
    """
    Old plotting routine no longer exists.
    """
    print '*** Error: plotclaw2html name is deprecated, use printframes.'
    print '*** Plotting command syntax has also changed, you may need to'
    print '*** update your setplot function.'

# -------------------------------------------------------------------------


#======================================================================
def pngfigs2html(plotdata,framenos=[0],frametimes=[0.],fignos=[1],fignames={}):
#======================================================================
    """
    take a sequence of figure files in format frame000NfigJ.png for
    N in framenos and J in fignos, and produce an html page for each
    along with an index page and a page showing all frames together for
    each figno and all figs together for each frameno.
    Dictionary frametimes[N] gives the time of the N'th frame.
    """

    numframes = len(framenos)
    numfigs = len(fignos)
    rootPath = os.getcwd()
    #fignos.sort()
    eagle = getattr(plotdata,'html_eagle',False)

    if len(framenos) == 0:
        print '*** Warning: 0 frames to print'
        msg = 'No html figure files generated'
        return msg
    if len(fignos) == 0:
        print '*** Warning: 0 figures to print each frame'
        msg = 'No html figure files generated'
        return msg

    pngfile = {}
    htmlfile = {}
    frametimef = {}
    allfigsfile = {}
    #print '    Making png and html files for %i frames:' % numframes, framenos
    for frameno in framenos:
        framef = string.zfill(frameno,4)
        ftime = frametimes[frameno]
        if ((ftime == 0) | ((ftime > 0.001) & (ftime < 1000))):
            ftimef = '%9.5f' % ftime
        else:
            ftimef = '%12.5e' % ftime
        frametimef[frameno] = ftimef
        framef = string.zfill(frameno,4)
        for figno in fignos:
            pngfile[frameno,figno] = 'frame%sfig%s.png'  % (framef,figno)
            htmlfile[frameno,figno] = 'frame%sfig%s.html' % (framef,figno)
        allfigsfile[frameno] = 'allfigsframe%s.html' % framef


    # determine current time and reformat:
    time1 = time.asctime()
    year = time1[-5:]
    day = time1[:-14]
    hour = time1[-13:-5]
    creationtime = day + year + ' at ' + hour
    
    
    # Write the index page
    
    # _PlotIndex.html
    #---------------
    
    html = open("_PlotIndex.html",'w')
    
    if eagle:
        html.write("""
  <html><meta http-equiv="expires" content="0">
  <title>EagleClaw Plot Index</title>
  <head>
  <link type="text/css" rel="stylesheet"
        href="http://localhost:50005/eagleclaw/eagleclaw.css">
  </head>
  <eagle1>EagleClaw -- Plot Index</eagle1>
  <eagle2>Easy Access Graphical Laboratory for Exploring Conservation
  Laws</eagle2>
  <p>
  <center><eagle3><a href="../eaglemenu.html">Main Menu for this run-directory
          </a></eagle3> </center><p>
        """)


    else:
        html.write('<html><meta http-equiv="expires" content="0">')
        html.write('<title>Clawpack Plot Index</title>')
        html.write('<body><center><h1>Plot Index</h1></center>\n' )
	homelink = getattr(plotdata,'html_homelink',None)
        if homelink:
	    html.write('<center><a href="%s">Back to %s</a></center>\n' \
	               % (homelink, homelink))

    html.write('<p>\n')
    html.write('<center>Plots created: %s &nbsp;&nbsp; ' % creationtime )
    #html.write(' On machine: %s\n'  % os.getenv('HOST'))
    html.write('</center><p>\n')

    html.write('<p>\n<table border=0 cellpadding=5 cellspacing=5>\n')


    if plotdata.latex_pdf:
        html.write('<p><tr><td><b>pdf file:</b></td>')
        html.write('\n   <td><a href="%s.pdf">%s.pdf</a></td>' \
               % (plotdata.latex_fname,plotdata.latex_fname))
        html.write('</tr>\n')

    if plotdata.html_movie:
        html.write('<p><tr><td><b>js Movies:</b></td>')
        for figno in fignos:
            html.write('\n   <td><a href="moviefig%s.html">%s</a></td>' \
                           % (figno,fignames[figno]))
        html.write('</tr>\n')
    if plotdata.gif_movie:
        html.write('<p><tr><td><b>gif Movies:</b></td>')
        for ifig in range(len(fignos)):
            html.write('\n   <td><a href="moviefig%s.gif">%s</a></td>' \
                           % (fignos[ifig],fignames[fignos[ifig]]))
        html.write('</tr>\n')
    html.write('<p>\n<tr><td><b>All Frames:</b></td> ')
    for ifig in range(len(fignos)):
        html.write('\n   <td><a href="allframesfig%s.html">%s</a></td>' \
                       % (fignos[ifig],fignames[fignos[ifig]]))
    html.write('</tr>\n')
    html.write('<p>\n<tr><td><b>Individual Frames:</b></td> </tr>\n')

    for frameno in framenos:

        html.write('\n <tr><td>Frame %s, t = %s:</td>' \
                    % (frameno,frametimef[frameno]))
        for ifig in range(len(plotdata._fignames)):
            figno = plotdata._fignos[ifig]
            figname = plotdata._fignames[ifig]
            html.write('\n   <td><a href="%s">%s</a></td>' \
                       % (htmlfile[frameno,figno],figname))
        if numfigs > 1:
            html.write('\n<td><a href="%s">All figures</a></td>' \
                       % allfigsfile[frameno])
        html.write('</tr>\n')
    html.write('</table>\n')
    html.write('</body></html>')

    #----------------------------------------------------------------------
    
    # allframesfigJ.html
    #-------------------
    for figno in fignos:
        html = open('allframesfig%s.html' % figno, 'w')
        html.write('<html><meta http-equiv="expires" content="0">')
        html.write('<title>Plots</title>')
        html.write('<body>\n<center><h1>All Frames -- %s</h1>\n' \
                 % fignames[figno])
        html.write('<p>\n')
        html.write('<h3><a href="_PlotIndex.html">Plot Index</a> </h3>\n')
        html.write('<p>\n')
        html.write('<h3>Click on a figure to enlarge</h3>\n')
        html.write('<p>\n')
    
        for frameno in framenos:
            html.write('  <a href="%s"><img src="%s" width=400></a>\n' \
                % (htmlfile[frameno,figno], pngfile[frameno,figno]))
    
        html.write('\n</center></body></html>\n')
        html.close()
    
    
    # allfigsframeN.html
    #-------------------
    if numfigs > 1:
        for iframe in range(numframes):
            frameno = framenos[iframe]
            html = open(allfigsfile[frameno], 'w')
            html.write('<html><meta http-equiv="expires" content="0">')
            html.write('<title>Plots</title>')
            html.write('<body>\n<center><h3>All Figures -- Frame %s' \
                 % framenos[iframe])
            html.write('&nbsp; at time t = %s' % frametimef[frameno])
            html.write('<p>\n')

            # Write link commands to previous and next frame:

            html.write('<p> <a href="%s">' % allfigsfile[framenos[0]])
            html.write('&#060; &#060;</a> &nbsp; &nbsp;\n')
            if iframe==0:
                html.write('&#060; &nbsp; &nbsp; ')
                html.write('\n<a href="_PlotIndex.html">Index</a>  ')
                if numframes > 1:
                    html.write('&nbsp; &nbsp; <a href="%s"> &#062; </a> ' \
                        % allfigsfile[framenos[1]])

            elif iframe==numframes-1:
                if numframes > 1:
                    html.write('\n<a href="%s"> &#060; </a>  &nbsp; &nbsp; ' \
                        % allfigsfile[framenos[iframe-1]])
                html.write('\n<a href="_PlotIndex.html">Index</a>  ')
                html.write(' &nbsp; &nbsp; &#062; ')

            else:
                html.write('\n<a href="%s"> &#060; </a>  &nbsp; &nbsp; ' \
                        % allfigsfile[framenos[iframe-1]])
                html.write('\n<a href="_PlotIndex.html">Index</a>  ')
                html.write('\n&nbsp; &nbsp; <a href="%s"> &#062; </a>  &nbsp; &nbsp; ' \
                        % allfigsfile[framenos[iframe+1]])

            html.write('&nbsp; &nbsp; \n<a href="%s"> ' \
                      % allfigsfile[framenos[numframes-1]])
            html.write('&#062; &#062;</a>  \n') 

            html.write('</h3><p>\n')
            html.write('<h3>Click on a figure to enlarge</h3>\n')
            html.write('<p>\n')
    
            for figno in fignos:
                html.write('  <a href="%s"><img src="%s" width=400></a>\n' \
                        % (htmlfile[frameno,figno], pngfile[frameno,figno]))

            # list of all frames at bottom:

            html.write('\n<p><b>Other frames:</b></a> &nbsp;&nbsp;')
            for frameno2 in framenos:
                if frameno2 == frameno:
                    html.write('\n<font color=red>%i</font>&nbsp;&nbsp;' \
                               % frameno)
                else:
                    html.write('\n<a href="%s">%i</a>  &nbsp; &nbsp; ' \
                           % (allfigsfile[frameno2],frameno2))
    
            html.write('\n</center></body></html>\n')
            html.close()
    
    
    # frameNfigJ.html  -- individual files for each frame/fig combo
    #----------------
    
    for iframe in range(numframes):
        frameno = framenos[iframe]
        for figno in fignos:
            html = open(htmlfile[frameno,figno],'w')
            html.write('<html><meta http-equiv="expires" content="0">\n')
            html.write('<title>Plots</title>')
            html.write('<body><center>\n')
            html.write('\n<h3>Frame %i ' % frameno)
            if numfigs > 1:
                html.write(' &nbsp;---&nbsp; %s' % fignames[figno] )
            html.write('&nbsp;&nbsp; at time t = %s</h3>' % frametimef[frameno])
        
            # Write link commands to previous and next frame:

            html.write('<p> <a href="%s">' % htmlfile[framenos[0],figno])
            html.write('&#060; &#060;</a> &nbsp; &nbsp;\n')
            if iframe==0:
                html.write('&#060; &nbsp; &nbsp; ')
                html.write('\n<a href="_PlotIndex.html">Index</a>  ')
                if numframes > 1:
                    html.write('&nbsp; &nbsp; <a href="%s"> &#062; </a> ' \
                        % htmlfile[framenos[1],figno])

            elif iframe==numframes-1:
                if numframes > 1:
                    html.write('\n<a href="%s"> &#060; </a>  &nbsp; &nbsp; ' \
                        % htmlfile[framenos[iframe-1],figno])
                html.write('\n<a href="_PlotIndex.html">Index</a>  ')
                html.write(' &nbsp; &nbsp; &#062; ')

            else:
                html.write('\n<a href="%s"> &#060; </a>  &nbsp; &nbsp; ' \
                        % htmlfile[framenos[iframe-1],figno])
                html.write('\n<a href="_PlotIndex.html">Index</a>  ')
                html.write('\n &nbsp; &nbsp;<a href="%s"> &#062; </a>  &nbsp; &nbsp; ' \
                        % htmlfile[framenos[iframe+1],figno])

            html.write('&nbsp; &nbsp; \n<a href="%s"> ' \
                      % htmlfile[framenos[numframes-1],figno])
            html.write('&#062; &#062;</a>  \n') 
        
            # image:
            html.write('\n\n <p><img src="%s"><p>  \n ' \
                        % pngfile[frameno,figno])

            html.write('\n\nImage source: &nbsp; %s'  \
                   % os.path.join(os.getcwd(),pngfile[frameno,figno]))

            # list of all frames at bottom:

            html.write('\n<p><b>Other frames:</b></a> &nbsp;&nbsp;')
            for frameno2 in framenos:
                if frameno2 == frameno:
                    html.write('\n<font color=red>%i</font>&nbsp;&nbsp;' \
                               % frameno)
                else:
                    html.write('\n<a href="%s">%i</a>  &nbsp; &nbsp; ' \
                           % (htmlfile[frameno2,figno],frameno2))
            html.write('\n<a href="allframesfig%s.html">  All Frames </a>' \
                     % figno)

            # list of all figures:

            if numfigs > 1:
                html.write('\n<p><b>Other figures at this time:</b> &nbsp;&nbsp;')
                for figno2 in fignos:
                    if figno2 == figno:
                        html.write('\n<font color=red>%s</font>&nbsp;&nbsp;' \
                               % fignames[figno])
                    else:
                        html.write('\n<a href="%s">%s</a>  &nbsp; &nbsp; ' \
                           % (htmlfile[frameno,figno2],fignames[figno2]))
                html.write('\n<a href="%s"> All Figures </a>' \
                     % allfigsfile[frameno])
        
            html.write('\n<p><h3><a href="_PlotIndex.html">Plot Index</a></h3>')
            if eagle:
                html.write("""<p><h3><a href="../eaglemenu.html">Main Menu for
                this run-directory</a></h3>  """)
            html.write('</center></body></html>')
            html.close()
    
    # moviefigJ.html
    #-------------------
    if plotdata.html_movie:
        for figno in fignos:
            html = open('moviefig%s.html' % figno, 'w')
            text = htmlmovie(pngfile,framenos,figno)
            html.write(text)
            html.close()
    
    
    PlotPath = os.getcwd()
    if PlotPath[0] != '/':
        PlotPath = '/' + PlotPath
    PlotPath.replace('\\','/') # for windows
    #print "Created html pages...  "
    msg = "\nPoint your browser to:\n"
    msg += "  file://" + PlotPath + "/_PlotIndex.html"
    clawdir = os.getenv('CLAW')
    if clawdir and (clawdir in PlotPath):
        PlotPath = PlotPath.replace(clawdir,'')
        msg += "\n\nOr, if you have the EagleClaw server running, point your browser to:"
        msg += "\n    http://localhost:50005" + PlotPath + "/_PlotIndex.html\n"
    return msg
    # end of pngfigs2html
    

def htmlmovie(pngfile,framenos,figno):
    text = """
<html>
<head>
<script language="Javascript">
<!---
var num_images = %s; """ % len(framenos)

    text += """
var img_width = 800;
var img_height = 600;
var interval = 300;
var images = new Array();

function fourdigits(i)
{
    if (i > 999)
        return i.toString();
    if (i > 99)
        return "0"+i.toString();
    if (i > 9)
        return "00"+i.toString();
    return "000"+i.toString()
}

function preload_images()
{
    t = document.getElementById("progress");
    """

    i = 0
    for frameno in framenos[1:]:
        i = i+1
        text += """
        t.innerHTML = "Preloading image ";
        images[%s] = new Image(img_width, img_height);
        images[%s].src = "%s";
        """ % (i,i,pngfile[frameno,figno])
    text += """
    t.innerHTML = "";
}

function tick()
{
  frame += 1;
  if (frame > num_images)
      frame = 1;

  document.movie.src = images[frame].src;
  setTimeout("tick()", interval);
}

function startup()
{
  preload_images();
  frame = 0;
  setTimeout("tick()", interval);
}
// --->
</script>
</head>
<body onLoad="startup();">
<p><div ID="progress">Please wait, preloading images...</div></p>
<img src="%s" name="movie"/>
</body>
</html>
""" % pngfile[framenos[0],figno]

    return text
    # end of htmlmovie


#======================================================================
def pngfigs2latex(plotdata,framenos=[0],frametimes=[0.],fignos=[1],fignames={}):
#======================================================================
    """
    take a sequence of figure files in format frame000NfigJ.png for
    N in framenos and J in fignos, and produce a latex file showing all. 
    Dictionary frametimes[N] gives the time of the N'th frame.
    Dictionary fignames[J] gives the name of the J'th fig for the index.
    """

    numframes = len(framenos)
    numfigs = len(fignos)
    rootPath = os.getcwd()

    if len(framenos) == 0:
        print '*** Warning: 0 frames to put in latex file'
        msg = 'No latex file generated'
        return msg
    if len(fignos) == 0:
        print '*** Warning: 0 figures to put in latex file'
        msg = 'No latex file generated'
        return msg
        
    pngfile = {}
    frametimef = {}
    for frameno in framenos:
        framef = string.zfill(frameno,4)
        ftime = frametimes[frameno]
        if ((ftime == 0) | ((ftime > 0.001) & (ftime < 1000))):
            ftimef = '%9.5f' % ftime
        else:
            ftimef = '%12.5e' % ftime
        frametimef[frameno] = ftimef
        framef = string.zfill(frameno,4)
        for figno in fignos:
            pngfile[frameno,figno] = 'frame%sfig%s.png'  % (framef,figno)

    for figno in fignos:
        if not fignames.has_key(figno):
            fignames[figno] = 'Figure %s' % figno

    # determine current time and reformat:
    time1 = time.asctime()
    year = time1[-5:]
    day = time1[:-14]
    hour = time1[-13:-5]
    creationtime = day + year + ' at ' + hour
    
    plotdir = plotdata.plotdir
    os.chdir(plotdir)
    
    latexfile = open('%s.tex' % plotdata.latex_fname, 'w')
    
    latexfile.write('''
\\documentclass[11pt]{article}
\\usepackage{graphicx}
\\setlength{\\textwidth}{7.5in}
\\setlength{\\oddsidemargin}{-0.5in}
\\setlength{\\evensidemargin}{-0.5in}
\\setlength{\\textheight}{9.2in}
\\setlength{\\voffset}{-1in}
\\setlength{\\headsep}{5pt}
\\begin{document}
\\centerline{\\Large\\bf CLAWPACK Results}\\vskip 5pt
    \n''')

    latexfile.write('\\centerline{\\bf Plots created {\\tt %s}' \
                        % (creationtime))
    latexfile.write(' in directory:} \n\\vskip 5pt')
    latexfile.write('\\hfil{\\verb+ %s+}\\hfil\n'  % (rootPath))
    latexfile.write('\\vskip 5pt\n')

    # determine how many plots should appear on each page and line:
    framesperpage = plotdata.latex_framesperpage 
    if framesperpage == 'all':
        framesperpage = len(framenos)
    framesperline = plotdata.latex_framesperline 
    if framesperline == 'all':
        framesperline = len(framenos)
    figsperline = plotdata.latex_figsperline      
    if figsperline == 'all':
        figsperline = len(fignos)
    if (figsperline < len(fignos)) & (framesperline > 1):
        print '*** Incompatible layout: resetting framesperline to 1'
        framesperline = 1
    totalperline = framesperline * figsperline      
    if totalperline < 1:
        print '*** Warning: 0 figures per line requested in latex file'
        msg = 'No latex file generated due to format error'
        return msg

    # width each plot must be:
    fwidth = 0.95/totalperline

    framecnt = 0
    for frameno in framenos:
        #latexfile.write('\\centerline{\Large Frame %s at time = %s' \
        #       % (frameno frametime[frameno])
        if framecnt >= framesperpage:
            latexfile.write('\\newpage \n')                       
            framecnt = 0
        elif framecnt >= framesperline:
            latexfile.write('\\vskip 10pt \n')                       
            framecnt = 0
        framecnt += 1
        figcnt = 0
        for figno in fignos:
            if figcnt >= figsperline:
                latexfile.write('\\vskip 10pt \n')                       
                figcnt = 0
            figcnt += 1
            latexfile.write('\\includegraphics[width=%s\\textwidth]{%s}\n' \
                            % (fwidth,pngfile[frameno,figno]))
        #latexfile.write('\\vskip 10pt \n')                       
    latexfile.write('\\end{document}\n')                         
    latexfile.close()
    msg = "Latex file created (use pdflatex to make pdf file):\n" 
    msg += "  %s/%s.tex" % (plotdir, plotdata.latex_fname)
    return msg
    # end of pngfigs2latex
    

#------------------------------------------------------------------

def var_limits(plotdata,vars,padding=0.1):
    """
    Determine range of values encountered in data for all frames
    in the most recent computation in order to determine appropriate
    y-axis limits for plots.

    vars is a list of variables to return the limits for.
    Each element of the list can be a number, indicating the 
    corresponding element of the q vector, or a function that should
    be applied to q to obtain the value of interest.  In other words,
    any valid plot_var attribute of a ClawPlotItem object.

    If vars=='all', use vars=[0,1,...] over all components of q.
    *** not implemented yet ***

    Returns: varmin, varmax, varlim,  
    each is a dictionary with keys consisting by the elements of vars.

    For each var in vars, 
      varmin[var] is the minimum of var over all frames,
      varmax[var] is the maximum of var over all frames,
      varlim[var] is of the form [v1, v2] suitable to use as the 
            ylim attritube of an object of class ClawPlotAxes with
                v1 = vmin - padding*(vmax-vmin)
                v2 = vmax + padding*(vmax-vmin)
            to give some space above and below the min and max.

    """

    varlim = {}
    vmin,vmax = var_minmax(plotdata,'all',vars)
    varmin = {}
    varmax = {}
    for var in vars:
        varmin[var] = vmin[var]['all']   # min over all frames
        varmax[var] = vmax[var]['all']   # max over all frames
        v1 = varmin[var] - padding*(varmax[var]-varmin[var])
        v2 = varmax[var] + padding*(varmax[var]-varmin[var])
        varlim[var] = [v1,v2]

    return varmin,varmax,varlim
        

#------------------------------------------------------------------

def var_minmax(plotdata,framenos,vars):
    """
    Determine range of values encountered in data for all frames
    in the list framenos.  If framenos='all', search over all frames
    in the most recent computation.

    vars is a list of variables to return the min and max for.
    Each element of the list can be a number, indicating the 
    corresponding element of the q vector, or a function that should
    be applied to q to obtain the value of interest.  In other words,
    any valid plot_var attribute of a ClawPlotItem object.

    If vars=='all', use vars=[0,1,...] over all components of q.
    *** not implemented yet ***

    Returns (varmin, varmax) where varmin and varmax are each 
    dictionaries with keys given by the variables.  

    For each var in vars, varmin[var] and varmax[var] are dictionaries, with
    keys given by frame numbers in framenos.
    
    Also varmin[var]['all'] and varmax[var]['all'] are the min and max of
    variable var over all frames considered.

    For example if vars = [0,'machnumber'] then
       varmin[0][3] is the minimum of q[0] (the first component of the
           solution vector q) over all grids in frame 3.
       varmin['machnumber']['all'] is the minimum of machnumber 
           over all grids in all frames.

    """

    framenos = only_most_recent(framenos, plotdata.outdir)
    if len(framenos) == 0:
        print '*** No frames found in var_minmax!'

    print 'Determining min and max of variables: ',vars
    print '   over frames: ',framenos

    varmin = {}
    varmax = {}
    if vars=='all':
        print "*** Error in var_minmax: vars == 'all' is not implemented yet"
        return (varmin,varmax)
    for var in vars:
        varmin[var] = {'all': inf}
        varmax[var] = {'all': -inf}
        for frameno in framenos:
            varmin[var][frameno] = inf
            varmax[var][frameno] = -inf

    for frameno in framenos:
        solution = plotdata.getframe(frameno)
        ndim = solution.ndim

        for igrid in range(len(solution.grids)):
            grid = solution.grids[igrid]
            for var in vars:
                if isinstance(var,int):
                    if ndim == 1:
                        qvar = grid.q[:,var]
                    elif ndim == 2:
                        qvar = grid.q[:,:,var]
                    elif ndim == 3:
                        qvar = grid.q[:,:,:,var]
                else:
                    t = solution.t
                    if ndim == 1:
                        qvar = var(grid.q, grid.xcenter, t)
                    elif ndim == 2:
                        qvar = var(grid.q, grid.xcenter, \
                                   grid.ycenter, t)
                    elif ndim == 3:
                        qvar = var(grid.q, grid.xcenter, \
                                   grid.ycenter, grid.zcenter, t)
                varmin[var][frameno] = min(varmin[var][frameno], qvar.min())
                varmax[var][frameno] = max(varmax[var][frameno], qvar.max())
                varmin[var]['all'] = min(varmin[var]['all'], \
                                         varmin[var][frameno])
                varmax[var]['all'] = max(varmax[var]['all'], \
                                         varmax[var][frameno])
    return (varmin, varmax)
        
#------------------------------------------------------------------


def only_most_recent(framenos,outdir='.',verbose=True):

    """
    Filter the list framenos of frame numbers and return a new
    list that only contains the ones from the most recent run.
    This is determined by comparing modification times of 
    fort.q files.

    Returns the filtered list.
    """

    import glob,time,os

    if outdir != '.':
        try:
            os.chdir(outdir)
        except:
            print "*** Could not chdir to ", outdir
            return framenos

    fortfile = {}
    for file in glob.glob('fort.q*'):
        frameno = int(file[7:10])
        fortfile[frameno] = file
    
    if len(fortfile) == 0:
        print '*** No fort.q files found in directory ', os.getcwd()
        framenos = []
        return framenos
    
    # Figure out which files are from latest run:
    numframes = 0
    mtime = 0
    framekeys = fortfile.keys()
    framekeys.sort()
    for frameno in framekeys:
        mtimeprev = mtime
        mtime = os.path.getmtime(fortfile[frameno])
        # sometimes later fort files are closed a few seconds after
        # earlier ones, so include a possible delaytime:
        delaytime = 5  # seconds
        if mtime < mtimeprev-delaytime:
            break
        numframes = numframes + 1
    
    newframes = framekeys[:numframes]
    if (numframes < len(framekeys)) & verbose:
        print '*** Frames %s and above appear to be from an old run' \
                       % framekeys[numframes]
        print '***    and will be ignored.'
        time.sleep(2)

    #print 'framenos = ',framenos
    if framenos == 'all':
        framenos = newframes
    else:
        # compute intersection of framenos and newframes:
        framenos = list(set(framenos).intersection(set(newframes)))
    framenos.sort()
    return framenos

#------------------------------------------------------------------------

def call_setplot(setplot, plotdata, verbose=True):
    """
    Try to apply setplot to plotdata and return the result.
    If setplot is False or None, return plotdata unchanged.
    If setplot is True, setplot function is in setplot.py.
    If setplot is a string, setplot function is in module named by string.
    Otherwise assume setplot is a function.
    """

    if not setplot:
        if verbose:
            print '*** Warning: no setplot specified'
        return plotdata

    if setplot is True:
        # indicates we should import setplot.py from current directory
        # and the setplot function is in this module.  
        try:
            sys.path.insert(0,os.getcwd())
            import setplot as SetPlot
            # make sure setplot from current directory is used:
            reload(SetPlot)
            setplot = SetPlot.setplot   # should be a function

            if verbose:
                print 'Imported setplot from ', os.getcwd()
        except:
            print """*** Error in call_setplot: 
                  Problem importing setplot.py in directory %s""" \
                  % os.getcwd()
            raise
            return plotdata

    elif isinstance(setplot,str):
        # assume setplot specifies module containing setplot function
        # strip off the .py if it is there:
        setplotmod = os.path.splitext(setplot)[0]
        try:
            sys.path.insert(0,os.getcwd())
            exec('import %s as SetPlot' % setplotmod)
            # make sure setplot from current directory is used:
            reload(SetPlot)
            setplot = SetPlot.setplot   # should be a function
        except:
            print """*** Error in call_setplot: 
                  Problem importing from %s.py in directory %s""" \
                  % (setplotmod, os.getcwd())
            raise
            return plotdata
    else:
        # assume setplot is the setplot function itself:
        pass

    # execute setplot:
    try:
        plotdata = setplot(plotdata)
        if verbose:
            print 'Executed setplot successfully'
    except:
        print '*** Error in call_setplot: Problem executing function setplot'
        raise
        return plotdata

    if plotdata is None:
        print '*** Error in printframes:  plotdata is None after call to setplot'
        print '*** Did you forget the "return plotdata" statement?'
        raise
        return plotdata
    
    return plotdata

def clawpack_header():
    axes([.3, .8, .98, 1.])
    text(.1,.13,'Clawpack Plots',fontsize=30,color='brown')
    axis('off')

def errors_2d_vs_1d(solution,reference,var_2d,var_1d,map_2d_to_1d):
    """
    Input: 
      solution: object of class ClawSolution with 2d computed solution

      reference: object of class ClawSolution with 1d reference solution

      var_2d: variable to compare from solution
      if integer, compare q[:,:,var_2d]
      if function, apply to q to obtain the variable

      var_1d: variable to compare from reference
      if integer, compare to q[:,var_2d]
      if function, apply to q to obtain the variable

      map_2d_to_1d: function mapping 2d data to 1d for comparison::

          xs, qs = map_2d_to_1d(xcenter, ycenter, q)
    """

    from numpy import interp

    t = solution.t
    xs = {}
    qs = {}
    qint = {}

    errmax = 0.
    for gridno in range(len(solution.grids)):
        grid = solution.grids[gridno]

        xc_center = grid.xcenter
        yc_center = grid.ycenter
        [X_center,Y_center] = meshgrid(xc_center,yc_center)

        if grid.mapc2p:
            try:
                X_center,Y_center = grid.mapc2p(X_center,Y_center)
            except:
                print "*** Error with mapc2p ***"
                print "*** in directory: ", os.getcwd()
                traceback.print_exc()
    
        if isinstance(var_2d, int):
            q = grid.q[:,:,var_2d]
        else:
            try:
                q = var_2d(grid.q, X_center, Y_center, t)
            except:
                print '*** Error applying function plot_var = ',plot_var
                traceback.print_exc()
                return 
        
	xs1, qs1 = map_2d_to_1d(q, X_center, Y_center, t)

	if hasattr(reference,'grids'):
	    if len(reference.grids) > 1:
	        print '*** Warning in errors_2d_vs_1d: reference solution'
	        print '*** has more than one grid -- only using grid[0]'
	    refgrid = reference.grids[0]
	else:
	    refgrid = reference   # assume this contains true solution or
	                          # something set separately rather than
				  # a framesoln
	xref = refgrid.xcenter
        if isinstance(var_1d, int):
            qref = refgrid.q[:,var_1d].T
        else:
            try:
                qref = var_1d(refgrid.q, xref, t)
            except:
                print '*** Error applying function var_1d'
                return 
	    
	qint1 = interp(xs1, xref, qref)

        xs[gridno] = xs1
        qs[gridno] = qs1
        qint[gridno] = qint1
	errabs = abs(qs1 - qint1)
	errmax = max(errmax, errabs.max())
	#import pdb
	#pdb.set_trace()

    return errmax, xs, qs, qint
        

def set_show(plotdata):
    """
    Determine which figures and axes should be shown.
    plotaxes._show should be true only if plotaxes.show and at least one
    item is showing.

    plotfigure._show should be gtrue only if plotfigure.show and at least
    one axes is showing.
    """

    for figname in plotdata._fignames:
        plotfigure = plotdata.plotfigure_dict[figname]
        plotfigure._show = False
        if plotfigure.show:
            # check to make sure some item is showing:
            for axesname in plotfigure._axesnames:
                plotaxes = plotfigure.plotaxes_dict[axesname]
                plotaxes._show = False
                if plotaxes.show:
                    # check to make sure some item is showing:
                    for itemname in plotaxes._itemnames:
                        plotitem = plotaxes.plotitem_dict[itemname]
                        plotitem._show = plotitem.show
                        if plotitem.show:
                            plotaxes._show = True
                            plotfigure._show = True
                
    return plotdata
