c =========================================================
      subroutine rp1(maxmx,meqn,mwaves,mbc,mx,ql,qr,auxl,auxr,
     &		     fwave,s,amdq,apdq)
c =========================================================
c
c     # begin_html  
c       [www.clawpack.org CLAWPACK] Riemann solver
c       for the 1D shallow water equations for axisymmetric flow on the 
c       sphere, u = meridional velocity, v = zonal velocity.
c
c       Uses the f-wave formulation.  No entropy fix implemented!
c       Parameters are set in [code: setprob.data] and read in by
c       [code: setprob.f].
c
c       For additional documentation on Riemann solvers rp1, see
c       [claw/doc/rp1]
c
c     # end_html
c
      implicit double precision (a-h,o-z)
      dimension   ql(1-mbc:maxmx+mbc, meqn)
      dimension   qr(1-mbc:maxmx+mbc, meqn)
      dimension    s(1-mbc:maxmx+mbc, mwaves)
      dimension fwave(1-mbc:maxmx+mbc, meqn, mwaves)
      dimension amdq(1-mbc:maxmx+mbc, meqn)
      dimension apdq(1-mbc:maxmx+mbc, meqn)
      dimension auxl(1-mbc:maxmx+mbc, *)
      dimension auxr(1-mbc:maxmx+mbc, *)
c
c     # local storage
c     ---------------
      dimension delta(3)
      logical efix
      common /param/ grav
      common /comsphere/ Rsphere, Omega
      common /comxlow/ xlower2
      data efix /.false./    !# use entropy fix for transonic rarefactions

      xlower = xlower2

c
      do 30 i=2-mbc,mx+mbc
c 
        xi = xlower+(i-1)*dx
        enx = dcos(xi)
        eny = dsin(xi)
        etx = -dsin(xi)
        ety = dcos(xi)

c
        hl = qr(i-1,1)
        hr = ql(i,1)
        hul = enx*qr(i-1,2)+eny*qr(i-1,3)
        hur = enx*ql(i,2)+eny*ql(i,3)

        hvl = etx*qr(i-1,2)+ety*qr(i-1,3)
        hvr = etx*ql(i,2)+ety*ql(i,3)
 
c     # compute  Roe-averaged quantities:
          ubar = (hul/dsqrt(qr(i-1,1))+hur/dsqrt(ql(i,1)))/
     &            (dsqrt(qr(i-1,1))+dsqrt(ql(i,1)))
          vbar = (hvl/dsqrt(qr(i-1,1))+hvr/dsqrt(ql(i,1)))/
     &            (dsqrt(qr(i-1,1))+dsqrt(ql(i,1)))
         cbar=dsqrt(0.5d0*grav*(qr(i-1,1) + ql(i,1)))

c     
      delta(1) = hur-hul
      delta(2) = hur**2./hr+0.5*grav*hr**2.
     &         - hul**2./hl-0.5*grav*hl**2. 
      delta(3) = hur*hvr/hr-hul*hvl/hl     
c
      hsl = auxr(i-1,1)
      hsr = auxl(i,1) 
      delta(2) = delta(2)+grav*0.5*(hl+hr)*(hsr-hsl)
  
c # compute coeffs in the evector expansion of delta(1),delta(2),delta(3)
      a1 = 0.5d0*(-delta(2) + (ubar + cbar) * delta(1))/cbar
      a2 = -vbar*delta(1)+delta(3)
      a3 = 0.5d0*( delta(2) - (ubar - cbar) * delta(1))/cbar

c     # finally, compute the waves.

         fwave(i,1,1) = a1
         fwave(i,2,1) = a1*(ubar-cbar)*enx + a1*vbar*etx
         fwave(i,3,1) = a1*(ubar-cbar)*eny + a1*vbar*ety
         s(i,1) = ubar - cbar

         fwave(i,1,2) = 0.d0
         fwave(i,2,2) = a2*etx
         fwave(i,3,2) = a2*ety          
         s(i,2) = ubar

         fwave(i,1,3) = a3
         fwave(i,2,3) = a3*(ubar+cbar) *enx + a3*vbar*etx
         fwave(i,3,3) = a3*(ubar+cbar) *eny + a3*vbar*ety
         s(i,3) = ubar + cbar

   30 continue

c     # compute Godunov flux f0:
c     --------------------------

      if (efix) go to 110

c     # no entropy fix
c     ----------------------------------------------
c     # amdq = SUM s*wave   over left-going waves
c     # apdq = SUM s*wave   over right-going waves

      do 100 m=1,meqn
         do 100 i=2-mbc, mx+mbc
            amdq(i,m) = 0.d0
            apdq(i,m) = 0.d0
            do 90 mw=1,mwaves
               if (s(i,mw) .lt. 0.d0) then
                   amdq(i,m) = amdq(i,m) + fwave(i,m,mw)
                 else
                   apdq(i,m) = apdq(i,m) + fwave(i,m,mw)
                 endif
   90          continue
  100       continue
c     project momentum components to tangent plane
      do i=2-mbc, mx+mbc
        xi = xlower+(i-1.5)*dx 
        enx = dcos(xi)
        eny = dsin(xi)
        etx = -eny
        ety = enx
        amn = etx*amdq(i,2)+ety*amdq(i,3)
        amdq(i,2) = amdq(i,2)-amn*etx
        amdq(i,3) = amdq(i,3)-amn*ety
c
        xi = xlower+(i-0.5)*dx 
        enx = dcos(xi)
        eny = dsin(xi)
        etx = -eny
        ety = enx
        apn = etx*apdq(i,2)+ety*apdq(i,3)
        apdq(i,2) = apdq(i,2)-apn*etx
        apdq(i,3) = apdq(i,3)-apn*ety 
      enddo 
      go to 900
c
c    -----------------------------------------------
c

  110 continue

      write(6,*) "Need to implement e-fix"
      stop
    
  900 continue

      do i=1-mbc,mx+mbc
         do mw=1,mwaves
            s(i,mw) = s(i,mw)/Rsphere
            enddo
         do m=1,meqn
            apdq(i,m) = apdq(i,m)/Rsphere
            amdq(i,m) = amdq(i,m)/Rsphere
            do mw=1, mwaves
              fwave(i,m,mw) = fwave(i,m,mw)/Rsphere
            enddo
         enddo
      enddo
      
      return
      end



