#
# Python script to run xclaw with several different parameter choices.
#

import os, shutil, glob
import time

# importing these modules requires subdirectory pyclaw:
from pyclaw import data, controller, error, plotting


def runjobs():
    clawjob = controller.ClawController()
    clawjob.xdir = os.getcwd()    # execute xclaw command from current dir.
    
    print "-----------------------------------------------------------------"
    
    t = {}
    err = {}
    tests = [[800,1],[50,3]]

    for itest in range(len(tests)):

        mx = tests[itest][0]
        mxnest = tests[itest][1]

        # directory for output:
        outdir = 'output.mx%smxnest%s' % (mx,mxnest) 

        if os.path.isdir(outdir):
            print "Found directory ",outdir
            ans = raw_input("OK to overwrite? ")
	    if ans.lower() not in ['y','yes']:
	        return
        else:
	    os.mkdir(outdir)

        print "Proceeding..."
	outdir = os.path.abspath(outdir)

        efile = open(outdir+'/errors.txt','w')

        # make local copies of data files to modify for each run:
        datafile = os.path.join(outdir,'amr2ez.data')
        shutil.copy('amr2ez_tests.data', datafile)
        shutil.copy('settsunami.data', outdir)
        shutil.copy('setgauges.data', outdir)
        shutil.copy('setprob.data', outdir)
        rundata = data.ClawRunData([datafile, 'ndim.data']) 
        rundata.mx = mx
        rundata.my = int(mx / 2)
        rundata.mxnest = -mxnest
        rundata.write(datafile)   # only changes amr2ez.data in outdir
        nout = rundata.nout

        # run fortran code:
        clawjob.rundir = outdir      # use data files from outdir
        clawjob.outdir = outdir      # write fort files to outdir
        clawjob.xclawcmd = 'xamr'    # executable for AMR
        tstart = time.time()
        returncode = clawjob.runxclaw()
        telapsed = time.time() - tstart
        if returncode != 0:
            print '*** returncode = ', returncode, '   aborting'
            return
        print 'Done executing with mx = %i' % mx
        print 'Elapsed time = %s' % telapsed
        print 'Output is in ', outdir
        efile.write('\n  elapsed time = %10.2f' % telapsed)
        
        
#------------------------------------

if __name__=='__main__':
    # when executed from command line, call main routine:
    runjobs()
    print "Next execute make_figure7.py"

