

# Set up the plots to be done for each frame

# This file is imported by the plotting routines and then
# setplot is called to set the plot parameters specifying what 
# is to be plotted and how the plots should be laid out.

from pylab import *
import os

refsoln = 'refsoln'   # directory for 1d reference solution
refsoln = os.path.abspath(refsoln)

#--------------------------
def setplot(plotdata):
#--------------------------
    """
    Argument plotdata should be an instance of pyclaw.data.ClawPlotData.
    This routine should return a modified plotdata.
    For documentation, see [claw:doc/plotting.html].
    """

    from pyclaw import colormaps
    from pyclaw.data import ClawPlotData
    from pyclaw.plotting import var_limits
    if not isinstance(plotdata,ClawPlotData):
        print '*** Error in setplot: plotdata must be a ClawPlotData object'
        return

    plotfigure = plotdata.new_plotfigure(name='Scatter plot', figno=6)

    plotaxes = plotfigure.new_plotaxes(name='Scatter')
    plotaxes.title = 'Scatter plot of depth'
    plotaxes.xlim = [-1., 1]
    plotaxes.ylim = [-2., 2]

    # scatter plot:
    plotitem = plotaxes.new_plotitem(name='scatter', \
                plot_type='1d_from_2d_data')
    plotitem.plot_var = 4

    # function mapping 2d data to 1d scatter plot data, defined below:
    plotitem.map_2d_to_1d = scatter_var   

    #plotitem.plotstyle = ['+', 'x', 'o', 's', '^']  # for different AMR levels
    plotitem.plotstyle = ['.', '.', '.', '.', '.']  # for different AMR levels
    plotitem.color = ['b','g','k','m','c']
    plotitem.kwargs = {'markersize': 8}

    # fine grid reference solution:
    plotitem = plotaxes.new_plotitem(name='1dref', plot_type='1d')
    plotitem.outdir = refsoln
    plotitem.plot_var = 3
    plotitem.plotstyle = '-'
    plotitem.color = 'r'
    plotitem.kwargs = {'linewidth': 2}

    plotdata.afterframe = print_error  # defined below
    return plotdata


#------------------------------------

def scatter_var(q,xcenter,ycenter,t):
    import mapc2m
    from numpy import arcsin
    Rsphere = 6.371e6
    X,Y,Z = mapc2m.mapc2m(xcenter,ycenter)
    r = arcsin(Z/Rsphere)
    latitude = r.flatten()
    qT = q.T
    qscatter = qT.flatten()
    return latitude, qscatter


#------------------------------------

def print_error(frameno, plotdata):
    errmax = compute_error(frameno,plotdata)
    print 'Max norm of error = %g '  % errmax
    

#------------------------------------

def compute_error(frameno, plotdata):
    from pyclaw.plotting import errors_2d_vs_1d

    solution = plotdata.getframe(frameno)
    reference = plotdata.getframe(frameno,refsoln)
    var_2d = 4  # sea surface in 2d calculation
    var_1d = 3  # sea surface in 1d calculation
    errmax,xs,qs,qint = errors_2d_vs_1d(solution,reference,var_2d, \
                         var_1d,scatter_var)
    return errmax
