c
c
c =========================================================
      subroutine src1d(meqn,mbc,mx1d,q1d,maux,aux1d,t,dt)
c =========================================================
      implicit double precision (a-h,o-z)
      dimension   q1d(mx1d, meqn)
      dimension aux1d(mx1d, maux)
      double precision RK(4,3)
      common /comsphere/ Rsphere, Omega
c
c
c     # This routine should be a simplified version of src2 
c     # which applies source terms for a 1-d slice of data along the
c     # edge of a grid.  This is called only from qad where the conservative
c     # fix-up is applied and is used to apply source terms over partial
c     # time steps to the coarse grid cell values used in solving Riemann 
c     # problems at the interface between coarse and fine grids.
c
c     # If the source terms depend only on q, it should be easy to 
c     # adapt src2 to create this routine, just loop over 1:mx1d.
c     # If the source terms are more complicated, it may not be easy.
c
c     # The code may work fine without applying source terms in this
c     # context, so using this dummy routine might be successful even when
c     # source terms are present. 
c
c    
      df = 2.d0*Omega

c     # No Coriolis term! #######################
c      return

      do i=1,mx1d
         erx = aux1d(i,14)
         ery = aux1d(i,15)
         erz = aux1d(i,16)
c
         xc = aux1d(i,17)
         yc = aux1d(i,18)
         call mapc2m(xc,yc,xp,yp,zp)
c         erx = xp 
c         ery = yp 
c         erz = zp 

         qn = erx*q1d(i,2) + ery*q1d(i,3) + erz*q1d(i,4)
       
         q1d(i,2) = q1d(i,2) - qn*erx
         q1d(i,3) = q1d(i,3) - qn*ery
         q1d(i,4) = q1d(i,4) - qn*erz
      enddo

      do i=1, mx1d
         xc = aux1d(i,17)
         yc = aux1d(i,18)
         call mapc2m(xc,yc,xp,yp,zp)
c         erx = xp 
c         ery = yp 
c         erz = zp 

         erx = aux1d(i,14)
         ery = aux1d(i,15)
         erz = aux1d(i,16)
c
         fcor = df*erz / (Rsphere**2.)

c        stage 1
         hu = q1d(i,2)
         hv = q1d(i,3)
         hw = q1d(i,4)
                              
         RK(1,1) = fcor*dt*(erz*hv-ery*hw)
         RK(1,2) = dt*fcor*(erx*hw-erz*hu)
         RK(1,3) = dt*fcor*(ery*hu-erx*hv)

c        stage 2
         hu = q1d(i,2) + 0.5d0*RK(1,1)
         hv = q1d(i,3) + 0.5d0*RK(1,2)
         hw = q1d(i,4) + 0.5d0*RK(1,3)

         RK(2,1) = fcor*dt*(erz*hv-ery*hw)
         RK(2,2) = dt*fcor*(erx*hw-erz*hu)
         RK(2,3) = dt*fcor*(ery*hu-erx*hv)

c        stage 3
         hu = q1d(i,2) + 0.5d0*RK(2,1)
         hv = q1d(i,3) + 0.5d0*RK(2,2)
         hw = q1d(i,4) + 0.5d0*RK(2,3)

         RK(3,1) = fcor*dt*(erz*hv-ery*hw)
         RK(3,2) = dt*fcor*(erx*hw-erz*hu)
         RK(3,3) = dt*fcor*(ery*hu-erx*hv)

c        stage 4
         hu = q1d(i,2) + 0.5d0*RK(3,1)
         hv = q1d(i,3) + 0.5d0*RK(3,2)
         hw = q1d(i,4) + 0.5d0*RK(3,3)

         RK(4,1) = dt*fcor*(erz*hv-ery*hw)
         RK(4,2) = dt*fcor*(erx*hw-erz*hu)
         RK(4,3) = dt*fcor*(ery*hu-erx*hv)
            
         do m=2,meqn
             q1d(i,m) = q1d(i,m) 
     &                 + (RK(1,m-1) + 2.d0*RK(2,m-1)+
     &                 2.d0*RK(3,m-1) + RK(4,m-1))/6.d0
         enddo
 
      enddo

      do i=1, mx1d
          erx = aux1d(i,14)
          ery = aux1d(i,15)
          erz = aux1d(i,16)
c
         xc = aux1d(i,17)
         yc = aux1d(i,18)
         call mapc2m(xc,yc,xp,yp,zp)
c         erx = xp 
c         ery = yp 
c         erz = zp 
          qn = erx*q1d(i,2) + ery*q1d(i,3) + erz*q1d(i,4)
c
          q1d(i,2) = q1d(i,2) - qn*erx
          q1d(i,3) = q1d(i,3) - qn*ery
          q1d(i,4) = q1d(i,4) - qn*erz
      enddo
      return
      end
